/**
    @addtogroup groupCore 
    @{
    @file config.h
    @brief Файл настройки проекта \b RTOS @osName.
    @details Файл используется для настройки проекта
    на базе операционной системы \b RTOS @osName.
    @see \ref sectionConfig.
    @}
*/

#ifndef _CONFIG_H_
#define _CONFIG_H_

/**
 *  @name Информация о проекте.
 *  @{
 */
/** Описание проекта для вывода на печать. */
#define PROJECT_BRIEF  "Multex-ARM Tasks test"
/** Номер версии - старшая часть. */
#define PROJECT_VERSION_NUMBER      1
/** Номер версии - младшая часть. */
#define PROJECT_VERSION_SUB_NUMBER  0
/// @}

#include <arch/archdef.h>

/**
 *  @brief Выбор процессора.
 *  @details Процессор должен быть определён для инициализации
 *  аппаратных модулей на ранней стадии запуска @osName.
 *  Следует выбирать имя процессора из соответствующей группы 
 *  \ref ahcArchProcNames "макросов".
 *  
 */
#define ARCH_PROC ARCH_PROC_V3S

/** Подключение сопроцессора \b NEON. */
#define INCLUDE_NEON

/** Подключение внутреннего кэша данных процессора. */
#define DCACHE_ENABLE

/** Подключение отладочной консоли \b DEBUG-UART (UART0). */
#define INCLUDE_SIO_CONSOLE

/** Подключение файловой системы \b MSDOS. */
#define INCLUDE_DOSFS

/** Подключение работы с \b USB. */
// #define INCLUDE_USB

/** Подключение модуля работы с \b Ethernet. */
#define INCLUDE_NETINET


#ifdef INCLUDE_NETINET
    #define IP_ADDRESS "10.0.3.35"           ///< Сетевой адрес устройства.
    #define CHECK_PRIMARY_IP_ADDRESS
    #define INCLUDE_NETLOADER
    #define INCLUDE_TCP                     ///< Активация стека \b TCP/IP.
    // #ifdef DEBUG
        #define FTP_SERVER                  ///< Активация \b FTP сервера для доступа к диску по сети.
        #define FTP_ALLOW_TO_CHANGE_WORK_DRIVE
        #define INCLUDE_NET_CONSOLE         ///< Активация подключения сетевой консоли.
        #define INCLUDE_TCP_CONSOLE         ///< Выбор протокола сетевой консоли.
    // #endif
#endif

//#define SM_CALL
//#define ETH_IS_GMAC
//#define DISABLE_EARLY_SERIAL_CONSOLE_OUTPUT
//#define KEYBOARD_DONT_USE_STDIN


/** Подключение процедуры пользователя, запускаемой сразу после загрузки ядра. */
// #define DRAW_LOGO testMsg

/** Подключение процедуры пользователя, запускаемой после загрузки ядра и всех драйверов. */
// #define USER_PROC allTest

#endif  //  _CONFIG_H_
