#include <multex.h>
#include <memlib.h>
#include <tasklib.h>
#include <gpio.h>
#include <semlib.h>
#include <msgqlib.h>

#define pin_A (P_G + 2)
#define pin_B (P_G + 3)

TASK_ID task_A = NULL, task_B = NULL;
SEM_ID sem = NULL;
MSG_Q_ID queue_A = NULL, queue_B = NULL;

volatile int value;
bool stop;

void AP (int priority) {
    taskPrioritySet (task_A, priority);
}

int testStop () {    
    stop = true;
    if (sem != NULL)
        semFlush (sem);
    
    taskDelay (1);

    if (queue_A != NULL)
        msgQDelete (queue_A);
    if (queue_B != NULL)
        msgQDelete (queue_B);
    return memAvail ();
}

void pulse (int pin) {
    gpio_set_value (pin, 1);
    for (int i = 0; i < 10000; i++) {
        value = (i + (value * value)) / 2;
    }
    gpio_set_value (pin, 0);
    for (int i = 0; i < 1000; i++) {
        value = (i + (value * value)) / 2;
    }
}

void halfPulse (int pin) {
    gpio_set_value (pin, 1);
    for (int i = 0; i < 5000; i++) {
        value = (i + (value * value)) / 2;
    }
    gpio_set_value (pin, 0);
    for (int i = 0; i < 1000; i++) {
        value = (i + (value * value)) / 2;
    }
}



//============================================
// Simple tasks

int taskSimple (int pin) {

    gpio_direction_output (pin, 0);

    while (!stop) {
        pulse (pin);
        taskDelay (1);
    }

    return 0;
}

void testSimple () {
    stop = false;
    task_A = taskSpawn ("task A", 10, 0, 0, taskSimple, pin_A);
    task_B = taskSpawn ("task B", 10, 0, 0, taskSimple, pin_B);
}











//============================================
// Semaphore tasks

int taskSem (int pin) {

    gpio_direction_output (pin, 0);

    while (!stop) {
        semTake (sem, WAIT_FOREVER);
        pulse (pin);
    }

    if (sem != NULL) {
        semDelete (sem);
        sem = NULL;
    }

    return 0;
}


void testSem () {
    stop = false;
    if (sem == NULL)
        sem = semBCreate (SEM_Q_FIFO, SEM_EMPTY);
    task_A = taskSpawn ("task A", 10, 0, 0, taskSem, pin_A);
    task_B = taskSpawn ("task B", 10, 0, 0, taskSem, pin_B);
}








int taskRelease (int val) {

    while (!stop) {
        if (sem != NULL) {
            semGive (sem);
            semGive (sem);
            semGive (sem);
            semGive (sem);
        }
        taskDelay (1);
    }

    return 0;
}



void testRelease () {
    taskSpawn ("task R", 15, 0, 0, taskRelease, 0);
}












//============================================
// Message tasks

typedef struct {
    int pin;
    MSG_Q_ID in;
    MSG_Q_ID out;
} tMsgDesc;

tMsgDesc msg_A, msg_B;

void setMsgDesc (tMsgDesc *desc, int pin, MSG_Q_ID in, MSG_Q_ID out) {
    desc->pin = pin;
    desc->in = in;
    desc->out = out;
}

int taskMsg (int addr) {
    tMsgDesc *desc = (tMsgDesc *) addr;
    gpio_direction_output (desc->pin, 0);

    while (!stop) {
        int data = value;

        while (msgQReceive (desc->in, &data, sizeof (int), 
            NO_WAIT) == OK);
        
        while (msgQSend (desc->out, &data, sizeof (int),
                      NO_WAIT, MSG_PRI_NORMAL) == OK) {
            pulse (desc->pin);
        }
        taskDelay (1);
    }
    return 0;
}

void testMsg () {
    stop = false;
    queue_A = msgQCreate (2, sizeof (int), MSG_Q_FIFO);
    queue_B = msgQCreate (2, sizeof (int), MSG_Q_FIFO);
    setMsgDesc (&msg_A, pin_A, queue_A, queue_B);
    setMsgDesc (&msg_B, pin_B, queue_B, queue_A);
    task_A = taskSpawn ("task A", 10, 0, 0, taskMsg, (int) &msg_A);
    task_B = taskSpawn ("task B", 10, 0, 0, taskMsg, (int) &msg_B);
}






